<?php
/**
 * The model file of invoice module of ZDOO.
 *
 * @copyright   Copyright 2009-2016 青岛易软天创网络科技有限公司(QingDao Nature Easy Soft Network Technology Co,LTD, www.cnezsoft.com)
 * @license     商业软件，非开源软件
 * @author      Tingting Dai <daitingting@xirangit.com>
 * @package     invoice 
 * @version     $Id$
 * @link        http://www.zdoo.com
 */
class invoiceModel extends model
{
    /**
     * Get invoice by id.
     * 
     * @param  int    $id 
     * @access public
     * @return object
     */
    public function getByID($id)
    {
        $invoice = $this->dao->select('*')->from(TABLE_INVOICE)->where('id')->eq($id)->fetch();
        if($invoice)
        {
            $taxRate = $invoice->taxRate / 100;
            $invoice->taxMoney = (float)($invoice->money / (1 + $taxRate) * $taxRate);
            $invoice->details  = $this->dao->select('*')->from(TABLE_INVOICEDETAIL)->where('invoice')->eq($invoice->id)->orderBy('id')->fetchAll('id');
            $invoice->files    = $this->loadModel('file')->getByObject('invoice', $id);

            /* Format date object to avoid create changes. */
            $invoice->createdDate     = formatTime(substr($invoice->createdDate, 0, 10));
            $invoice->editedDate      = formatTime(substr($invoice->editedDate, 0, 10));
            $invoice->receivedDate    = formatTime(substr($invoice->receivedDate, 0, 10));
            $invoice->checkedDate     = formatTime(substr($invoice->checkedDate, 0, 10));
            $invoice->drawnDate       = formatTime(substr($invoice->drawnDate, 0, 10));
            $invoice->expressedDate   = formatTime(substr($invoice->expressedDate, 0, 10));
            $invoice->taxRefundedDate = formatTime(substr($invoice->taxRefundedDate, 0, 10));
        }
        return $invoice;
    }

    /**
     * Get invoice list.
     * 
     * @param  string $kind 
     * @param  string $mode 
     * @param  int    $company 
     * @param  string $month 
     * @param  string $status 
     * @param  int    $customerID 
     * @param  int    $contractID 
     * @param  string $orderBy 
     * @param  object $pager 
     * @access public
     * @return array 
     */
    public function getList($kind = '', $mode = '', $company = '', $month = '', $status = '', $customerID = 0, $contractID = 0, $orderBy = 'id_desc', $pager = null)
    {
        $date = '';
        if(strlen($month) == 4) $date = $month;
        if(strlen($month) == 6) $date = substr($month, 0, 4) . '-' . substr($month, 4, 2);

        $customerIdList = array();
        $customerIdList = $this->loadModel('customer')->getCustomersSawByMe();
        if(empty($customerIdList)) return array();

        /* process search condition. */
        if($this->session->invoiceQuery == false) $this->session->set('invoiceQuery', ' 1 = 1');
        $invoiceQuery = $this->loadModel('search', 'sys')->replaceDynamic($this->session->invoiceQuery);

        $invoiceList = $this->dao->select('*')->from(TABLE_INVOICE)
            ->where('deleted')->eq('0')
            ->andWhere('customer')->in($customerIdList)
            ->beginIF($kind)->andWhere('kind')->eq($kind)->fi()
            ->beginIF($date)->andWhere('createdDate')->like("{$date}-%")->fi()
            ->beginIF($company)->andWhere('company')->eq($company)->fi()
            ->beginIF($status)->andWhere('status')->eq($status)->fi()
            ->beginIF($customerID)->andWhere('customer')->eq($customerID)->fi()
            ->beginIF($contractID)->andWhere('contract')->eq($contractID)->fi()
            ->beginIF($mode == 'bysearch')->andWhere($invoiceQuery)->fi()
            ->orderBy($orderBy)
            ->page($pager)
            ->fetchAll('id');

        $this->session->set('invoiceQueryCondition', $this->dao->get());

        $detailList = $this->dao->select('*')->from(TABLE_INVOICEDETAIL)
            ->where('invoice')->in(array_keys($invoiceList))
            ->orderBy('id')
            ->fetchGroup('invoice');

        foreach($invoiceList as $invoice)
        {
            $taxRate = $invoice->taxRate / 100;
            $invoice->taxMoney = (float)($invoice->money / (1 + $taxRate) * $taxRate);
            $invoice->details  = zget($detailList, $invoice->id, array()); 
        }
        return $invoiceList;
    }

    /**
     * Get my invoice id list.
     * 
     * @param  string $type   view|edit
     * @param  array  $invoiceIdList 
     * @access public
     * @return array
     */
    public function getInvoicesSawByMe($type = 'view', $invoiceIdList = array())
    {
        $customerIdList = $this->loadModel('customer')->getCustomersSawByMe($type);
        $invoiceList = $this->dao->select('*')->from(TABLE_INVOICE)
            ->where('deleted')->eq('0')
            ->beginIF(!empty($invoiceIdList))->andWhere('id')->in($invoiceIdList)->fi()
            ->beginIF(!isset($this->app->user->rights['crm']['manageall']) and ($this->app->user->admin != 'super'))
            ->andWhere('customer')->in($customerIdList)
            ->fi()
            ->fetchAll('id');

        return array_keys($invoiceList);
    }

    /**
     * Get related trades of the invoice.
     * 
     * @param  int    $invoiceID 
     * @access public
     * @return array
     */
    public function getRelatedTrades($invoiceID)
    {
        return $this->dao->select('trade')->from(TABLE_TRADEINVOICE)->where('invoice')->eq($invoiceID)->fetchPairs();
    }

    /**
     * Get trades to relate with invoice.
     * 
     * @param  int    $customerID 
     * @param  int    $contractID 
     * @param  array  $trades 
     * @access public
     * @return array
     */
    public function getTrades($type, $customerID, $contractID = 0, $invoiceTrades = array())
    {
        $trades = $this->dao->select('*')->from(TABLE_TRADE)->alias('t1')
            ->leftJoin(TABLE_TRADEINVOICE)->alias('t2')->on('t1.id=t2.trade')
            ->where('t1.type')->eq($type)
            ->andWhere('t1.trader')->eq($customerID)
            ->beginIF(!empty($invoiceTrades))->orWhere('t2.trade')->in($invoiceTrades)->fi()
            ->fetchAll();

        if($contractID)
        {
            $contractTrades = array();
            foreach($trades as $key => $trade)
            {
                if($trade->contract == $contractID or isset($invoiceTrades[$trade->id])) $contractTrades[] = $trade;
            }
            if($contractTrades) $trades = $contractTrades;
        }

        $this->app->loadLang('trade', 'cash');

        $tradePairs = array();
        $depositors = $this->loadModel('depositor', 'cash')->getPairs();
        foreach($trades as $trade)
        {
            $tradePairs[$trade->id] = $trade->date . zget($depositors, $trade->depositor) . zget($this->lang->trade->typeList, $type) . zget($this->lang->currencySymbols, $trade->currency) . $trade->money;
        }
        return $tradePairs;
    }

    /**
     * Get month money.
     * 
     * @param  int     $company 
     * @param  string  $year 
     * @access public
     * @return array
     */
    public function getMonthMoney($company, $year)
    {
        return $this->dao->select("DATE_FORMAT(drawnDate, '%m') months, sum(money) as totalMoney")->from(TABLE_INVOICE)
            ->where('status')->eq('normal')
            ->andWhere('company')->eq($company)
            ->andWhere('drawnDate')->like("{$year}%")
            ->groupBy('months')
            ->fetchPairs();
    }

    /**
     * Get company pairs used in invoice module.
     *
     * @access public
     * @return array
     */
    public function getInvoiceCompanies()
    {
        $companies = $this->dao->select('DISTINCT company')->from(TABLE_INVOICE)->fetchPairs();

        return $this->loadModel('company')->getPairs('all', $companies);
    }

    /**
     * Get invoice count of a customer or a contract.
     *
     * @param  string $type
     * @param  mixed  $param int | array
     * @access public
     * @return int | array
     */
    public function getInvoiceCount($type, $param = 0)
    {
        if(is_array($param))
        {
            return $this->dao->select("{$type}, COUNT(*) AS count")
                ->from(TABLE_INVOICE)
                ->where('deleted')->eq('0')
                ->andWhere($type)->in($param)
                ->groupBy($type)
                ->fetchPairs();
        }

        return $this->dao->select('COUNT(*) AS count')
            ->from(TABLE_INVOICE)
            ->where('deleted')->eq('0')
            ->andWhere($type)->eq($param)
            ->fetch('count');
    }

    /**
     * Get company pairs include the passed companyID.
     *
     * @param  int    $companyID
     * @access public
     * @return array
     */
    public function getCompanies($companyID = 0)
    {
        $companies = $this->loadModel('company')->getPairs();
        if($companyID && !isset($companies[$companyID]))
        {
            $company = $this->company->getByID($companyID);
            $companies[$company->id] = $company->name;
        }

        return $companies;
    }

    /**
     * Get serial number of a invoice.
     *
     * @param  int    $company
     * @param  string $drawnDate
     * @access public
     * @return int
     */
    public function getSerialNumber($company, $drawnDate)
    {
        $drawnMonth   = substr($drawnDate, 0, 8);
        $serialNumber = $this->dao->select('MAX(serialNumber) AS serialNumber')->from(TABLE_INVOICE)
            ->where('company')->eq($company)
            ->andWhere('drawnDate')->like("{$drawnMonth}%")
            ->fetch('serialNumber');
        return $serialNumber + 1;
    }

    /**
     * Create a invoice;
     * 
     * @access public
     * @return bool
     */
    public function create()
    {
        $invoice = fixer::input('post')
            ->add('status', 'wait')
            ->add('createdBy', $this->app->user->account)
            ->add('createdDate', helper::now()) 
            ->setDefault('contract', 0)
            ->setDefault('taxRate', 0)
            ->setForce('money', (float)$this->post->money)
            ->setIF($this->post->kind == 'sale', 'status', 'wait')
            ->setIF($this->post->kind == 'purchase', 'status', 'normal')
            ->stripTags('desc', $this->config->allowedTags)
            ->remove('itemList,itemTypeList,itemIDList,modelList,unitList,amountList,priceList,moneyList,trades')
            ->remove('customerInvoice,drawnMoney,files,labels,continue,hasTrade,uid')
            ->get();

        /* Check details not empty. */
        if($this->post->kind == 'sale')
        {
            $detailEmpty = true;
            foreach($this->post->itemIDList as $key => $itemID)
            {
                if(!empty($itemID) and !empty($this->post->moneyList[$key]))
                {
                    $detailEmpty = false;
                    break;
                }
            }
            if($detailEmpty)
            {
                $itemEmpty   = true;
                $priceEmpty  = true;
                $amountEmpty = true;
                foreach($this->post->itemIDList as $key => $itemID)
                {
                    if(!empty($itemID)) $itemEmpty = false;
                    if(!empty((float)$this->post->priceList[$key])) $priceEmpty = false;
                    if(!empty((float)$this->post->amountList[$key])) $amountEmpty = false;
                }
                if($itemEmpty)   dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->item);
                if($priceEmpty)  dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->price);
                if($amountEmpty) dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->amount);
            }

            if($invoice->type == 'companySpecial') $this->config->invoice->require->create .= ',registedAddress,phone,bankName,bankAccount';
        }

        $this->dao->insert(TABLE_INVOICE)
            ->data($invoice)
            ->autoCheck()
            ->batchCheck($this->config->invoice->require->create, 'notempty');

        if(dao::isError()) return false;

        $this->dao->exec();
        $invoiceID = $this->dao->lastInsertID();

        if(!empty($_POST['uid'])) $this->loadModel('file')->updateObjectID($this->post->uid, $invoiceID, 'invoice');

        if(!$this->saveDetails($invoiceID, $invoice->taxRate)) 
        {
            $this->delete(TABLE_INVOICE, $invoiceID);
            return false;
        }

        if($this->post->hasTrade)
        {
            $trades = $this->post->trades ? $this->post->trades : array();
            $this->updateTradeInvoice($invoiceID, $trades);
        }

        if($this->post->kind == 'sale') $this->updateCustomerInvoice($invoice);

        return $invoiceID;
    }

    /**
     * Update invoice.
     * 
     * @param  int    $invoiceID 
     * @access public
     * @return bool|array
     */
    public function update($invoiceID)
    {
        $oldInvoice = $this->getByID($invoiceID);

        $invoice = fixer::input('post')
            ->add('editedBy', $this->app->user->account)
            ->add('editedDate', helper::now())
            ->setDefault('customer', $oldInvoice->customer)
            ->setDefault('contract', 0)
            ->setForce('money', (float)$this->post->money)
            ->setIF($this->post->type != 'companyDigitalOrdinary', 'sendway', 'express')
            ->stripTags('desc', $this->config->allowedTags)
            ->remove('idList,itemList,itemTypeList,itemIDList,modelList,unitList,amountList,priceList,moneyList,files,labels,hasTrade,trades,uid')
            ->get();
        
        $requiredFields = $this->config->invoice->require->edit;

        if($oldInvoice->kind == 'sale')
        {
            /* Check details not empty. */
            $detailEmpty = true;
            foreach($this->post->itemIDList as $key => $itemID)
            {
                if(!empty($itemID) and !empty((float)$this->post->moneyList[$key]))
                {
                    $detailEmpty = false;
                    break;
                }
            }

            if($detailEmpty)
            {
                $itemEmpty   = true;
                $priceEmpty  = true;
                $amountEmpty = true;
                foreach($this->post->itemIDList as $key => $itemID)
                {
                    if(!empty($itemID)) $itemEmpty = false;
                    if(!empty((float)$this->post->priceList[$key])) $priceEmpty = false;
                    if(!empty((float)$this->post->amountList[$key])) $amountEmpty = false;
                }
                if($itemEmpty)   dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->item);
                if($priceEmpty)  dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->price);
                if($amountEmpty) dao::$errors['itemList'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->amount);
            }

            if($invoice->status != 'wait') 
            {
                $requiredFields .= ',sn,drawnBy,drawnDate';
                if($invoice->status != 'normal')
                {
                    $requiredFields .= ',taxRefundedBy,taxRefundedDate';
                }
                if($this->post->expressedBy or $this->post->expressedDate)
                {
                    $requiredFields .= ',contact,sendway';
                    if($this->post->expressedBy)                   $requiredFields .= ',expressedDate';
                    if($this->post->expressedDate)                 $requiredFields .= ',expressedBy';
                    if($invoice->type != 'companyDigitalOrdinary') $requiredFields .= ',express,waybill,address';
                    if($invoice->type == 'companyDigitalOrdinary') $requiredFields .= ',sendAccount';
                }
            }
            if($invoice->type == 'companySpecial') $requiredFields .= ',registedAddress,phone,bankName,bankAccount';
        }
        
        $this->dao->update(TABLE_INVOICE)->data($invoice)
            ->where('id')->eq($invoiceID)
            ->autoCheck()
            ->batchCheck($requiredFields, 'notempty');

        if(dao::isError()) return false;

        if(!empty($_POST['uid'])) $this->loadModel('file')->updateObjectID($this->post->uid, $invoiceID, 'invoice');

        $this->dao->exec();
        
        if(!$this->saveDetails($invoiceID, $invoice->taxRate)) return false;

        $trades = ($this->post->hasTrade && $this->post->trades) ? $this->post->trades : array();
        $this->updateTradeInvoice($invoiceID, $trades);

        if($oldInvoice->kind == 'sale') $this->updateCustomerInvoice($invoice);

        return commonModel::createChanges($oldInvoice, $invoice);
    }

    /**
     * Save details of an invoice. 
     * 
     * @param  int    $invoiceID 
     * @param  int    $taxRate
     * @access public
     * @return bool
     */
    public function saveDetails($invoiceID, $taxRate)
    {
        $this->dao->delete()->from(TABLE_INVOICEDETAIL)->where('invoice')->eq($invoiceID)->exec();

        /* Save details. */
        if(!empty($_POST['itemList']))
        {
            $itemList     = array();
            $itemPairs    = $this->getItemPairs();
            $productPairs = $this->loadModel('product')->getPairs();
            foreach($productPairs as $productID => $productName) $itemList["product{$productID}"] = $productName;
            foreach($itemPairs as $itemID => $itemName) $itemList["item{$itemID}"] = $itemName;

            foreach($this->post->itemList as $key => $item)
            {
                if(!$item or !$this->post->itemIDList[$key] or empty($this->post->moneyList[$key])) continue;

                $detail = new stdclass();
                $detail->invoice  = $invoiceID;
                $detail->item     = zget($itemList, $item);
                $detail->itemType = $this->post->itemTypeList[$key];
                $detail->itemID   = $this->post->itemIDList[$key];
                $detail->model    = $this->post->modelList[$key];
                $detail->unit     = $this->post->unitList[$key];
                $detail->price    = $this->post->priceList[$key];
                $detail->amount   = $this->post->amountList[$key];
                $detail->money    = (float)$this->post->moneyList[$key];
                $detail->taxRate  = $taxRate;
                $detail->taxMoney = (float)($detail->money / (1 + ($taxRate / 100)) * ($taxRate / 100));

                $this->dao->insert(TABLE_INVOICEDETAIL)->data($detail)->autoCheck()->exec();
            }
        }

        return !dao::isError();
    }

    /**
     * Drawn an invoice.
     * 
     * @param  int    $invoiceID 
     * @access public
     * @return bool
     */
    public function drawn($invoiceID)
    {
        $oldInvoice = $this->getByID($invoiceID);

        $invoice = fixer::input('post')
            ->add('status', 'normal')
            ->remove('files,labels')
            ->get();

        $this->dao->update(TABLE_INVOICE)->data($invoice)
            ->where('id')->eq($invoiceID)
            ->autoCheck()
            ->batchCheck($this->config->invoice->require->drawn, 'notempty')
            ->exec();

        if(dao::isError()) return false;

        if($oldInvoice->serialNumber == 0)
        {
            $invoice->serialNumber = $this->getSerialNumber($oldInvoice->company, $invoice->drawnDate);
            $this->dao->update(TABLE_INVOICE)->set('serialNumber')->eq($invoice->serialNumber)->where('id')->eq($invoiceID)->exec();
        }
         
        $fileAction = '';

        $files = $this->loadModel('file')->saveUpload('invoice', $invoiceID);
        if($files)
        {
            $this->dao->update(TABLE_FILE)->set('extra')->eq('e-invoice')
                ->where('id')->in(array_keys($files))
                ->andWhere('extension')->in('jpg,jpeg,png,bmp,pdf')
                ->exec();
            $fileAction = $this->lang->addFiles . join(',', $files);
        }

        $changes = commonModel::createChanges($oldInvoice, $invoice);
        $extra   = array('crm.invoice', 'view', "invoiceID=$invoiceID", $this->lang->invoice->sn . ' ' . $invoice->sn);
        $action  = $this->loadModel('action')->create('invoice', $invoiceID, 'Drawn', $fileAction);
        if($changes) $this->action->logHistory($action, $changes);

        $this->action->create('customer', $oldInvoice->customer, 'drawnInvoice', $fileAction, $extra);
        if($oldInvoice->contract) $this->action->create('contract', $oldInvoice->contract, 'drawnInvoice', $fileAction, $extra);

        return true;
    }

    /**
     * Express an invoice. 
     * 
     * @param  int    $invoiceID 
     * @access public
     * @return bool
     */
    public function express($invoiceID)
    {
        $oldInvoice = $this->getByID($invoiceID);

        $invoice = fixer::input('post')
            ->add('expressedBy', $this->app->user->account)
            ->add('expressedDate', helper::now())
            ->setIF($oldInvoice->type != 'companyDigitalOrdinary', 'sendway', 'express')
            ->remove('createAddress,newAddress,createContact,newContact,contactPhone,files,labels,subject,content,attachments,continue')
            ->get();

        if($oldInvoice->type == 'companyDigitalOrdinary')
        {
            $errors = array();
            if(!$this->post->sendAccount) $errors['sendAccount'] = sprintf($this->lang->error->notempty, $this->lang->invoice->sendwayList[$this->post->sendway]);
            if($this->post->sendway == 'email')
            {
                if(!validater::checkEmail($this->post->sendAccount)) $errors['sendAccount'] = sprintf($this->lang->error->email, $this->post->sendAccount);
                if(!$this->post->subject) $errors['subject'] = sprintf($this->lang->error->notempty, $this->lang->invoice->subject);
                if(!$this->post->content) $errors['content'] = sprintf($this->lang->error->notempty, $this->lang->invoice->content);
            }
            if($errors) return array('result' => 'fail', 'message' => $errors);
        }

        /* Create contact for current customer. */
        if($this->post->createContact)
        {
            $invoice->contact = 0;

            if($this->post->newContact)
            {
                $contact = new stdclass();
                $contact->realname = $this->post->newContact;
                $contact->phone    = $this->post->contactPhone;
                $contact->customer = $oldInvoice->customer;

                if(strpos(',email,qq,weixin,', ",{$invoice->sendway},") !== false) $contact->{$invoice->sendway} = $this->post->sendAccount;

                $return = $this->loadModel('contact', 'crm')->create($contact);
                if(isset($return['result']) && $return['result'] == 'fail') return $return;

                $invoice->contact = $return['contactID'];
            }
        }

        /* Create address for current customer. */
        if($this->post->createAddress)
        {
            $invoice->address = 0;

            if($this->post->newAddress)
            {
                $address = new stdclass();
                $address->objectType = 'customer';
                $address->objectID   = $oldInvoice->customer;
                $address->title      = $this->lang->invoice->address;
                $address->location   = $this->post->newAddress;

                $this->dao->insert(TABLE_ADDRESS)->data($address)->autoCheck()->exec();
                if(dao::isError()) return false;

                $invoice->address = $this->dao->lastInsertId();
            }
        }

        if($oldInvoice->type != 'companyDigitalOrdinary' && empty($invoice->address)) dao::$errors['newAddress'][] = sprintf($this->lang->error->notempty, $this->lang->invoice->address);

        $this->dao->update(TABLE_INVOICE)->data($invoice)
            ->where('id')->eq($invoiceID)
            ->batchCheck($this->config->invoice->require->express, 'notempty')
            ->autoCheck()
            ->exec();

        return commonModel::createChanges($oldInvoice, $invoice);
    }

    /**
     * Refund tax for an invoice.
     * 
     * @param  int     $invoiceID 
     * @param  string  $status 
     * @access public
     * @return bool
     */
    public function taxRefund($invoiceID, $status)
    {
        $oldInvoice = $this->getByID($invoiceID);
        
        $invoice = new stdclass();
        $invoice->status          = $status;
        $invoice->taxRefundedBy   = $this->app->user->account;
        $invoice->taxRefundedDate = helper::now();

        $this->dao->update(TABLE_INVOICE)->data($invoice)->where('id')->eq($invoiceID)->autoCheck()->exec();

        if(dao::isError()) return false;

        $changes = commonModel::createChanges($oldInvoice, $invoice);
        $action  = $status == 'canceled' ? 'canceledInvoice' : 'redInvoice';
        $extra   = array('crm.invoice', 'view', "invoiceID=$invoiceID", $this->lang->invoice->sn . ' ' . $oldInvoice->sn);

        $actionID = $this->loadModel('action')->create('invoice', $invoiceID, ($status == 'canceled' ? 'Canceled' : 'Red'), $this->post->comment);
        if($changes) $this->action->logHistory($actionID, $changes);

        $this->action->create('customer', $oldInvoice->customer, $action, $this->post->comment, $extra);
        if($oldInvoice->contract) $this->action->create('contract', $oldInvoice->contract, $action, $this->post->comment, $extra);

        return true;
    }

    /**
     * Update customer invoice. 
     * 
     * @param  object $invoice 
     * @access public
     * @return bool
     */
    public function updateCustomerInvoice($invoice)
    {
        $oldInvoice = $this->dao->select('*')->from(TABLE_CUSTOMERINVOICE)
            ->where('customer')->eq($invoice->customer)
            ->andWhere('taxNumber')->eq($invoice->taxNumber)
            ->fetch();

        $newInvoice = new stdclass();
        $newInvoice->customer = $invoice->customer;
        foreach($this->config->invoice->fields as $field) $newInvoice->$field = $invoice->$field;

        $this->loadModel('action');
        if($oldInvoice)
        {
            $this->dao->update(TABLE_CUSTOMERINVOICE)->data($newInvoice)
                ->where('id')->eq($oldInvoice->id)
                ->autoCheck()
                ->exec();

            $changes = commonModel::createChanges($oldInvoice, $newInvoice);
            if($changes)
            {
                $action = $this->action->create('customerInvoice', $oldInvoice->id, 'edited');
                $this->action->logHistory($action, $changes);
            }
        }
        else
        {
            $this->dao->insert(TABLE_CUSTOMERINVOICE)->data($newInvoice)->autoCheck()->exec();
            $id = $this->dao->lastInsertId();
            $this->action->create('customerInvoice', $id, 'created');
        }

        return !dao::isError();
    }

    /**
     * Update trade invoice.
     * 
     * @param  int    $invoiceID 
     * @param  array  $trades 
     * @access public
     * @return bool
     */
    public function updateTradeInvoice($invoiceID, $trades = array())
    {
        /* Delete current invoiceID first before updating the plan invoices. */
        $oldPlanList = $this->dao->select('id, trade, invoice')->from(TABLE_PLAN)->where('invoice')->like("%{$invoiceID}%")->fetchAll();
        foreach($oldPlanList as $oldPlan)
        {
            $oldPlan->invoice = ',' . $oldPlan->invoice . ',';
            $tmpInvoice = trim(',', str_replace(",{$invoiceID}", '', $oldPlan->invoice));
            $this->dao->update(TABLE_PLAN)->set('invoice')->eq($tmpInvoice)->where('id')->eq($oldPlan->id)->exec();
        }

        /* Delete current tradeInvoice first before updating tradeInvoice. */
        $this->dao->delete()->from(TABLE_TRADEINVOICE)->where('invoice')->eq($invoiceID)->exec();

        if(empty($trades)) return !dao::isError();

        /* Get plans in current trades. */
        $planList = $this->dao->select('id, trade, invoice')->from(TABLE_PLAN)->where('trade')->in($trades)->fetchAll();

        $tradeInvoice = new stdclass();
        $tradeInvoice->invoice     = $invoiceID;
        $tradeInvoice->createdBy   = $this->app->user->account;
        $tradeInvoice->createdDate = helper::now();
        foreach($trades as $tradeID)
        {
            $tradeInvoice->trade = $tradeID;
            $this->dao->insert(TABLE_TRADEINVOICE)->data($tradeInvoice)->autoCheck()->exec();

            foreach($planList as $plan)
            {
                if($plan->trade == $tradeID)
                {
                    $planInvoice = !empty($plan->invoice) ? $plan->invoice . ',' . $invoiceID : $invoiceID;
                    $this->dao->update(TABLE_PLAN)->set('invoice')->eq($planInvoice)->where('id')->eq($plan->id)->exec();
                }
            }
        }

        return !dao::isError();
    }

    /**
     * Export invoices.
     *
     * @param  string $type     detail | customer
     * @access public
     * @return void
     */
    public function export($type)
    {
        $invoiceList    = array();
        $queryCondition = $this->session->invoiceQueryCondition;
        if($queryCondition)
        {
            if(strpos($queryCondition, 'LIMIT') !== false) $queryCondition = substr($queryCondition, 0, strpos($queryCondition, 'LIMIT'));

            $stmt = $this->dbh->query($queryCondition);
            while($row = $stmt->fetch()) $invoiceList[$row->id] = $row;
        }

        $func = 'export' . ucfirst($type);
        $this->$func($invoiceList);
    }

    /**
     * Export invoice details to excel. 
     * 
     * @param  array  $invoiceList 
     * @access public
     * @return void
     */
    public function exportDetail($invoiceList)
    {
        $fields = array();
        $rows   = array();

        foreach($this->config->invoice->excel->fields as $field)
        {
            $fields[$field] = $this->lang->invoice->$field;
        }

        $companyList  = $this->loadModel('company')->getPairs();
        $customerList = $this->loadModel('customer')->getPairs();
        $depositors   = $this->loadModel('depositor', 'cash')->getPairs();
        $tradeList    = $this->loadModel('trade', 'cash')->getList('in');
        $itemList     = $this->loadModel('product')->getPairs() + $this->getItemPairs();
        $returnList   = $this->dao->select('*')->from(TABLE_PLAN)->fetchGroup('contract');
        $detailList   = $this->dao->select('*')->from(TABLE_INVOICEDETAIL)
            ->where('invoice')->in(array_keys($invoiceList))
            ->orderBy('id')
            ->fetchGroup('invoice');

        $i = 0;
        foreach($invoiceList as $invoice)
        {
            $rows[$i]['company']   = zget($companyList, $invoice->company, '');
            $rows[$i]['drawnDate'] = formatTime($invoice->drawnDate, DT_DATE1);
            $rows[$i]['customer']  = zget($customerList, $invoice->customer, '');
            $rows[$i]['type']      = zget($this->lang->invoice->typeList, $invoice->type, '');
            $rows[$i]['sn']        = $invoice->sn;
            $rows[$i]['status']    = zget($this->lang->invoice->statusList, $invoice->status, '');
            $rows[$i]['money']     = $invoice->money;
            $rows[$i]['desc']      = $invoice->desc; 

            $returnMoney   = 0;
            $returnDetails = '';
            $item          = '';

            $returns = zget($returnList, $invoice->contract, '');
            if($returns)
            {
                foreach($returns as $key => $return)
                {
                    $depositor = '';
                    foreach($tradeList as $trade)
                    {
                        if($trade->contract == $return->contract && $trade->money = $return->amount && $trade->date = $return->returnedDate)
                        {
                            $depositor = zget($depositors, $trade->depositor, '');
                        }
                    }

                    if($key > 0) $returnDetails .= "\n";
                    $returnDetails .= $return->returnedDate . ' ' . $return->amount . ' ' . $depositor;
                    $returnMoney   += $return->amount;
                }
            }

            if(isset($detailList[$invoice->id]))
            {
                foreach($detailList[$invoice->id] as $detail)
                {
                    if(isset($itemList[$detail->item])) $item .= $itemList[$detail->item];
                }
            }

            $rows[$i]['returnMoney']   = $returnMoney;
            $rows[$i]['returnDetails'] = $returnDetails;
            $rows[$i]['item']          = $item;

            $i++;
        }

        $rows = $this->processExportData($rows);

        $data = new stdclass();
        $data->fields       = $fields;
        $data->kind         = 'invoice';
        $data->rows         = $rows;
        $data->title        = $this->lang->excel->title->invoice;
        $data->numberFields = $this->config->invoice->excel->numberFields; 
        $data->customWidth  = $this->config->invoice->excel->customWidth;

        $excelData = new stdclass();
        $excelData->dataList = array($data);
        $excelData->fileName = $this->post->fileName ? $this->post->fileName : $this->lang->excel->title->invoice;

        $this->app->loadClass('excel')->export($excelData, $this->post->fileType);
    }

    /**
     * Export customers to xml. 
     * 
     * @param  array  $invoiceList 
     * @access public
     * @return void
     */
    public function exportCustomer($invoiceList)
    {
        $clientLang  = $this->app->getClientLang();
        $zfile       = $this->app->loadClass('zfile');
        $cacheRoot   = $this->app->getCacheRoot();
        $customerDir = $cacheRoot . 'customer';
        if(file_exists($customerDir)) $zfile->removeDir($customerDir);
        $zfile->mkDir($customerDir);

        $pinyin    = $this->app->loadClass('pinyin');
        $default   = $this->convertChinese($this->lang->invoice->default);
        $companies = $this->loadModel('company')->getPairs();

        $invoices = array();
        foreach($invoiceList as $invoice)
        {
            if(!$invoice->company or !isset($companies[$invoice->company])) continue;

            $company = $companies[$invoice->company];
            $date    = substr($invoice->createdDate, 0, 11);
            $invoices[$company][$date][] = $invoice;
        }

        $companyCount = count($invoices);
        foreach($invoices as $company => $companyInvoices)
        {
            $categoryIndex = 1;
            $categoryXml   = "<FENLEI>\n";
            $customerXml   = "<KHXX>\n";
            foreach($companyInvoices as $date => $dateInvoices)
            {
                $bm = $categoryIndex;
                if($categoryIndex < 100) $bm = 1 . $categoryIndex;
                if($categoryIndex < 10)  $bm = 10 . $categoryIndex;
                $categoryXml .= "<Row BM='{$bm}' PID='0' O_BM='{$bm}' MC='{$date}' O_PID='0'/>\n";

                $customerIndex = 1;
                foreach($dateInvoices as $invoice)
                {
                    $code      = $bm . str_pad($customerIndex, 5, '0', STR_PAD_LEFT);
                    $customer  = $this->process($invoice->invoiceTitle);
                    $bank      = $this->process($invoice->bankName);
                    $account   = $this->process($invoice->bankAccount);
                    $address   = $this->process($invoice->registedAddress);
                    $phone     = $this->process($invoice->phone);
                    $taxNumber = $this->process($invoice->taxNumber);

                    $yhzh = '';
                    $dz   = '';
                    $jm   = '';

                    if($bank && $account)  $yhzh = $bank . ' ' . $account;
                    if($address && $phone) $dz   = $address . ' ' . $phone;

                    $pinyins = $pinyin->convert($customer);
                    foreach($pinyins as $p) if($p) $jm .= strtoupper($p[0]);

                    $customerXml .= "<Row BM='$code' PID='{$bm}' DZ='$dz' O_BM='$code' JM='$jm' KZ1='' NSRSBH='{$taxNumber}' MC='$customer' YHZH='$yhzh' YJDZ='' O_PID='{$bm}'/>\n";

                    $customerIndex++;
                }
                $categoryIndex++;
            }
            $categoryXml .= "</FENLEI>\n";
            $customerXml .= "</KHXX>\n";

            $xml  = "<?xml version='1.0' encoding='GB18030'?>\n";
            $xml .= "<Data TYPE='KEHUBIANMA'>\n";
            $xml .= $categoryXml;
            $xml .= $customerXml;
            $xml .= '</Data>';

            $fileName = $company . '.xml';
            if($companyCount > 1) $fileName = $this->convertChinese($fileName);
            file_put_contents($customerDir . DIRECTORY_SEPARATOR . $fileName, $xml);
        }

        $file  = '';
        $files = glob($customerDir . DIRECTORY_SEPARATOR . '*.xml');
        if(count($files) == 1)
        {
            $file = reset($files);
        }
        else
        {
            $this->app->loadClass('pclzip', true);
            helper::cd($cacheRoot);
            $zip = new pclzip("customer.zip");
            $zip->create('customer');

            $file = $cacheRoot . 'customer.zip';
        }

        /* basename function is incompatible with chinese named file. */
        $fileName = substr($file, strrpos($file, '/')+1);
        $fileSize = filesize($file);

        /* Set the downloading cookie, thus the export form page can use it to judge whether to close the window or not. */
        setcookie('downloading', 1);

        /* urlencode the filename for ie. */
        $isIE11 = strpos($this->server->http_user_agent, 'Trident') !== false and strpos($this->server->http_user_agent, 'rv:11.0') !== false; 
        if(strpos($this->server->http_user_agent, 'MSIE') !== false or $isIE11) $fileName = urlencode($fileName);

        header('Content-Description: File Transfer');
        header('Content-type: application/octet-stream');
        header("Content-Disposition: attachment; filename=\"$fileName\"");
        header("Content-length: {$fileSize}");
        header('Pragma: no-cache');
        header('Expires: 0');

        readfile($file);

        $zfile->removeFile($file);
        $zfile->removeDir($customerDir);

        exit;
    }

    /**
     * Export files of digital invoices.
     *
     * @param  int    $invoiceList
     * @access public
     * @return void
     */
    public function exportDigital($invoiceList)
    {
        $clientLang = $this->app->getClientLang();
        $zfile      = $this->app->loadClass('zfile');
        $cacheRoot  = $this->app->getCacheRoot();
        $invoiceDir = $cacheRoot . 'invoice';
        if(is_dir($invoiceDir)) $zfile->removeDir($invoiceDir);
        $zfile->mkDir($invoiceDir);

        $this->loadModel('file');
        $companies = $this->loadModel('company')->getPairs('all');
        foreach($invoiceList as $invoice)
        {
            if($invoice->status == 'wait') continue;
            if($invoice->type != 'companyDigitalOrdinary') continue;

            $files = $this->file->getByObject('invoice', $invoice->id);
            if(!$files) continue;

            $dirName = substr($invoice->drawnDate, 5, 2); 
            if(count($companies) > 1)
            {
                $dirName = zget($companies, $invoice->company) . DIRECTORY_SEPARATOR . $dirName;
                $dirName = $this->convertChinese($dirName);
            }

            $fileDir = $invoiceDir . DIRECTORY_SEPARATOR . $dirName;
            if(!is_dir($fileDir)) $zfile->mkDIR($fileDir);

            $index = 1;
            $count = count($files);
            foreach($files as $file)
            {
                $fileName = $invoice->serialNumber . ($count > 1 ? '_' . $index : '') . '.' . $file->extension; 
                $filePath = $fileDir . DIRECTORY_SEPARATOR . $fileName;
                $pathName = $this->file->getRealPathName($file->pathname);
                $realPath = $this->file->savePath . $pathName;

                $zfile->copyFile($realPath, $filePath);

                $index++;
            }
        }

        $fileName = 'invoice.zip';
        $this->app->loadClass('pclzip', true);
        helper::cd($cacheRoot);
        $zip = new pclzip($fileName);
        $zip->create('invoice');

        $file     = $cacheRoot . $fileName;
        $fileSize = filesize($file);

        /* Set the downloading cookie, thus the export form page can use it to judge whether to close the window or not. */
        setcookie('downloading', 1);

        /* urlencode the filename for ie. */
        $isIE11 = strpos($this->server->http_user_agent, 'Trident') !== false and strpos($this->server->http_user_agent, 'rv:11.0') !== false; 
        if(strpos($this->server->http_user_agent, 'MSIE') !== false or $isIE11) $fileName = urlencode($fileName);

        header('Content-Description: File Transfer');
        header('Content-type: application/octet-stream');
        header("Content-Disposition: attachment; filename=\"$fileName\"");
        header("Content-length: {$fileSize}");
        header('Pragma: no-cache');
        header('Expires: 0');

        readfile($file);

        $zfile->removeFile($file);
        $zfile->removeDir($invoiceDir);

        exit;
    }

    /**
     * Remove unicode characters.
     *
     * @param  string $input
     * @access public
     * @return string
     */
    public function process($input)
    {
        $pattern = "/[\x{007f}-\x{009f}]|\x{00a0}|\x{00ad}|[\x{0483}-\x{0489}]|[\x{0559}-\x{055a}]|\x{058a}|[\x{0591}-\x{05bd}]|\x{05bf}|[\x{05c1}-\x{05c2}]|[\x{05c4}-\x{05c7}]|[\x{0606}-\x{060a}]|[\x{063b}-\x{063f}]|\x{0674}|[\x{06e5}-\x{06e6}]|\x{070f}|[\x{076e}-\x{077f}]|\x{0a51}|\x{0a75}|\x{0b44}|[\x{0b62}-\x{0b63}]|[\x{0c62}-\x{0c63}]|[\x{0ce2}-\x{0ce3}]|[\x{0d62}-\x{0d63}]|\x{135f}|[\x{200b}-\x{200f}]|[\x{2028}-\x{202e}]|\x{2044}|\x{2071}|[\x{f701}-\x{f70e}]|[\x{f710}-\x{f71a}]|\x{fb1e}|[\x{fc5e}-\x{fc62}]|\x{feff}|\x{fffc}/u";
        $input   = preg_replace($pattern, '', trim($input));

        return $this->convertChinese($input);
    }

    /**
     * Convert string to GB18030.
     *
     * @param  string $input
     * @access public
     * @return string
     */
    public function convertChinese($input)
    {
        $clientLang = $this->app->getClientLang();
        if($clientLang != 'zh-cn' && $clientLang != 'zh-tw') return $input;

        $output     = $input;
        $toEncoding = $clientLang == 'zh-cn' ? 'GB18030' : 'BIG5';
        if(function_exists('mb_convert_encoding'))
        {
            $output = @mb_convert_encoding($input, $toEncoding, 'utf-8');
        }              
        elseif(function_exists('iconv'))
        {
            $output = @iconv('utf-8', $toEncoding, $input);
        }

        return $output;
    }

    /**
     * Get item pairs.
     * 
     * @access public
     * @return array
     */
    public function getItemPairs()
    {
        return $this->dao->select('id, value')->from(TABLE_LANG)->where('module')->eq('invoice')->andWhere('section')->eq('itemList')->fetchPairs();
    }

    /**
     * Get report data. 
     * 
     * @param  int    $company
     * @param  string $month 
     * @access public
     * @return void
     */
    public function getReportData($company = 0, $month = '')
    {
        $date = '';
        if(strlen($month) == 4) $date = $month;
        if(strlen($month) == 6) $date = substr($month, 0, 4) . '-' . substr($month, 4, 2);

        $invoices = $this->dao->select("company, DATE_FORMAT(createdDate, '%m') AS month, status, SUM(money) AS money")
            ->from(TABLE_INVOICE)
            ->where('deleted')->eq('0')
            ->andWhere('company')->ne('0')
            ->beginIF($company)->andWhere('company')->eq($company)->fi()
            ->beginIF($date)->andWhere('createdDate')->like("$date-%")->fi()
            ->groupBy("company, month, status")
            ->fetchAll();
        
        $data = array();
        if(strlen($month) == 6)
        {
            foreach($invoices as $invoice) $data[$invoice->month][$invoice->company][$invoice->status] = $invoice->money;
        }
        else
        {
            foreach($invoices as $invoice) $data[$invoice->company][$invoice->month][$invoice->status] = $invoice->money;
        }

        /* Compute total data. */
        foreach($data as $k => $kData)
        {
            foreach($kData as $j => $jData)
            {
                foreach($jData as $status => $money)
                {
                    if(isset($data[$k]['total'][$status]))
                    {
                        $data[$k]['total'][$status] += $money;
                    }
                    else
                    {
                        $data[$k]['total'][$status] = $money;
                    }
                }
            }
        }

        return $data;
    }

    /**
     * Get all months. 
     * 
     * @access public
     * @return array
     */
    public function getAllMonths($company = '')
    {
        $allMonths = array();
        $monthList = $this->dao->select("DISTINCT DATE_FORMAT(createdDate, '%Y%m') AS month")->from(TABLE_INVOICE)
            ->where('deleted')->eq('0')
            ->andWhere('company')->ne('0')
            ->beginIF($company)->andWhere('company')->eq($company)->fi()
            ->orderBy('createdDate')
            ->fetchPairs();
        foreach($monthList as $year)
        {
            $month = substr($year, 4, 2);
            $year  = substr($year, 0, 4);
            if(!$year) $year = date('Y');
            
            if($month)
            {
                $allMonths[$year][$month] = $month;
            }
            else
            {
                $allMonths[$year] = array();
            }
        }
        return $allMonths;
    }

    /**
     * Get module menu for invoice.
     *
     * @param  string $kind
     * @param  int    $company
     * @param  string $month
     * @param  string $status
     * @param  array  $monthList
     * @access public
     * @return string
     */
    public function getModuleMenu($kind = 'sale', $company = 0, $month = '', $status = '', $monthList = array())
    {
        $methodName = 'browse' . ucfirst($kind);

        $moduleMenu = "<nav id='menu'><div class='container'><ul class='nav nav-default'>";

        $hasAllBrowsePriv = true;
        foreach($this->lang->invoice->kindList as $kindKey => $kindName)
        {
            if(!commonModel::hasPriv('invoice', "browse{$kindKey}"))
            {
                $hasAllBrowsePriv = false;
                break;
            }
        }

        if($hasAllBrowsePriv)
        {
            $moduleMenu .= "<li class='dropdown active'>";
            $moduleMenu .= html::a('javascript:;', zget($this->lang->invoice->kindList, $kind) . "<span class='caret'></span>", "class='dropdown-toggle' data-toggle='dropdown'");
            $moduleMenu .= "<ul role='menu' class='dropdown-menu'>";
            foreach($this->lang->invoice->kindList as $kindKey => $kindName)
            {
                if(commonModel::hasPriv('invoice', "browse{$kindKey}")) $moduleMenu .= commonModel::printLink('invoice', "browse{$kindKey}", '', $kindName, '', false, '', 'li');
            }
            $moduleMenu .= "</ul></li>";
        }
        else
        {
            foreach($this->lang->invoice->kindList as $kindKey => $kindName)
            {
                if(commonModel::hasPriv('invoice', "browse{$kindKey}"))
                {
                    $moduleMenu .= "<li class='active'>" . commonModel::printLink('invoice', "browse{$kindKey}", '', $kindName, '', false) . '</li>';
                }
            }
        }

        /* All menu. */
        $class = ($company || $month || $status) ? '' : 'active';
        $moduleMenu .= "<li class='$class'>" . commonModel::printLink('invoice', "browse{$kind}", "mode=all", $this->lang->invoice->all, '', false) . '</li>';

        /* Company menus. */
        $companies = $this->getInvoiceCompanies();
        if(!empty($companies))
        {
            $class = $company ? 'active' : '';
            $moduleMenu .= "<li class='$class dropdown'>";
            $moduleMenu .= html::a('javascript:;', zget($companies, $company, $this->lang->invoice->company) . "<span class='caret'></span>", "class='dropdown-toggle' data-toggle='dropdown'");
            $moduleMenu .= "<ul role='menu' class='dropdown-menu'>";
            foreach($companies as $companyID => $currentCompany)
            {
                $moduleMenu .= commonModel::printLink('invoice', "browse{$kind}", "mode=&company=$companyID&month=$month&status=$status", $currentCompany, '', false, '', 'li');
            }
            $moduleMenu .= "</ul></li>";
        }

        /* Month menus. */
        $currentYear  = substr($month, 0, 4);
        $currentMonth = substr($month, 4, 2);
        if(!$currentYear) $currentYear = date('Y');
        if(!$monthList) $monthList = $this->getAllMonths();
        if(!empty($monthList))
        {
            $suffix = ($this->app->getClientLang() == 'zh-cn' or $this->app->getClientLang() == 'zh-tw') ? $this->lang->year : '';
            $class  = $month ? 'active' : '';
            $moduleMenu .= "<li class='$class dropdown'>";
            $moduleMenu .= html::a('javascript:;', $currentYear . $suffix . "<span class='caret'></span>", "class='dropdown-toggle' data-toggle='dropdown'");
            $moduleMenu .= "<ul role='menu' class='dropdown-menu'>";
            ksort($monthList);
            foreach($monthList as $year => $months)
            {
                $moduleMenu .= commonModel::printLink('invoice', "browse{$kind}", "mode=&company=$company&month=$year&status=$status", $year . $suffix, '', false, '', 'li');
            }
            $moduleMenu .= "</ul></li>";

            if(isset($monthList[$currentYear]) && $monthList[$currentYear])
            {
                $class = $currentMonth ? 'active' : '';
                $moduleMenu .= "<li class='$class dropdown'>";
                $moduleMenu .= html::a('javascript:;', zget($this->lang->invoice->monthList, $currentMonth, $this->lang->invoice->month) . "<span class='caret'></span>", "class='dropdown-toggle' data-toggle='dropdown'");
                $moduleMenu .= "<ul role='menu' class='dropdown-menu'>";
                krsort($monthList[$currentYear]);
                foreach($monthList[$currentYear] as $currentMonth)
                {
                    if(!$currentMonth) continue;
                    $moduleMenu .= commonModel::printLink('invoice', "browse{$kind}", "mode=&company=$company&month=$currentYear$currentMonth&status=$status", $this->lang->invoice->monthList[$currentMonth], '', false, '', 'li');
                }
                $moduleMenu .= "</ul></li>";
            }
        }

        /* Status menus. */
        $class = $status ? 'active' : '';
        $moduleMenu .= "<li class='$class dropdown'>";
        $moduleMenu .= html::a('javascript:;', zget($this->lang->invoice->statusList, $status, $this->lang->invoice->status) . "<span class='caret'></span>", "class='dropdown-toggle' data-toggle='dropdown'");
        $moduleMenu .= "<ul role='menu' class='dropdown-menu'>";
        foreach($this->lang->invoice->statusList as $key => $currentStatus)
        {
            if($kind == 'purchase' && ($key == 'red' or $key == 'wait')) continue;

            $moduleMenu .= commonModel::printLink('invoice', "browse{$kind}", "mode=&company=$company&month=$month&status=$key", $currentStatus, '', false, '', 'li');
        }
        $moduleMenu .= "</ul></li>";

        /* Report menu. */
        if($kind == 'sale')
        {
            $moduleName = $this->app->getModuleName();
            $methodName = $this->app->getMethodName();
            $class = ($moduleName == 'invoice' && $methodName == 'report') ? "class='active'" : '';
            $moduleMenu .= "<li $class>" . commonModel::printLink('invoice', 'report', "company=$company&month=$month", $this->lang->invoice->report->common, '', false) . '</li>';
        }

        $moduleMenu .= "</ul></div></nav>";
        return $moduleMenu;
    }

    /**
     * Check if an invoice is clickable.
     *
     * @param  string $module
     * @param  string $method
     * @param  object $invoice
     * @access public
     * @return bool
     */
    public function isClickable($module, $method, $invoice)
    {
        if($module == 'invoice' && $method == 'delete')    return $invoice->kind == 'sale' && $invoice->status == 'wait';
        if($module == 'invoice' && $method == 'drawn')     return $invoice->kind == 'sale' && $invoice->status == 'wait';
        if($module == 'invoice' && $method == 'express')   return $invoice->kind == 'sale' && $invoice->status == 'normal' && !$invoice->expressedBy;
        if($module == 'invoice' && $method == 'taxRefund') return $invoice->status == 'normal';

        return true;
    }

    /**
     * Build operate menu.
     *
     * @param  object $invoice
     * @param  string $type
     * @access public
     * @return string
     */
    public function buildOperateMenu($invoice, $type = 'browse')
    {
        $menu  = '';
        $class = $type == 'browse' ? '' : 'btn';
        $label = $invoice->type != 'companyDigitalOrdinary' ? $this->lang->invoice->express : $this->lang->invoice->send;
        $menu .= $this->buildMenu('invoice', 'drawn', "invoiceID={$invoice->id}", $this->lang->invoice->drawn, $invoice, $type, "class='{$class}' data-toggle='modal'");
        $menu .= $this->buildMenu('invoice', 'express', "invoiceID={$invoice->id}", $label, $invoice, $type, "class='{$class}' data-toggle='modal'");
        $menu .= $this->buildMenu('invoice', 'taxRefund', "invoiceID={$invoice->id}&status=canceled", $this->lang->invoice->cancel, $invoice, $type, "class='{$class}' data-toggle='modal'");

        if($invoice->kind == 'sale') $menu .= $this->buildMenu('invoice', 'taxRefund', "invoiceID={$invoice->id}&status=red", $this->lang->invoice->red, $invoice, $type, "class='{$class}' data-toggle='modal'");

        $menu .= $this->buildMenu('invoice', 'edit',   "invoiceID={$invoice->id}", $this->lang->edit,   $invoice, $type, "class='{$class}'");
        $menu .= $this->buildMenu('invoice', 'delete', "invoiceID={$invoice->id}", $this->lang->delete, $invoice, $type, "class='{$class} deleter'");
        $menu .= $this->buildFlowMenu('invoice', $invoice, $type);

        return $menu;
    }
}
